﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using AIOUSBNet;  // the namespace exposes the AIOUSB Class interface 

// You must also add a reference to the AIOUSBnet.dll in the project settings
// Usually intalled in C:\Windows\system32


namespace Sample1
{
    public partial class Form1 : Form
    {
        // One and only Device Index:
        public UInt32 DeviceIndex;

        // AD specifics:
        const int Channels = 16;

        // Struct for AI Board Config:
        public struct TUSBAI1616Config
        {
            public TUSBAI1616Config(int size)
            {
                ChannelRange = new byte[size];
                CalibMode = 0;
                TrigMode = 0;
                StartStopCH = 0;
                Oversample = 0;
                AIMUXChEx = 0;
            }

            public byte[] ChannelRange;

            public byte CalibMode;
            public byte TrigMode;
            public byte StartStopCH;
            public byte Oversample;
            public byte AIMUXChEx; //AIMUX start and end address bit extension register.
        }

        // Num channels we will be using:
        public const int numAnalogOutputs = 16;

        // Struct for Configuration must pass number of channels for array size:
        public TUSBAI1616Config Config = new TUSBAI1616Config(numAnalogOutputs);
        
        // Required for ADC_SetConfig() dll calls:
        UInt32 ConfigBufSize = numAnalogOutputs + 5; // sizeof(Config); //could be better

        // Temp array for ADC_SetConfig() dll calls:
        public byte[] configArray = new byte[21];

        // Array for AD Data the counts are unsigned 16 bit numbers (ushort)
        //public UInt16[] ADData = new UInt16[16 * 32 * 1024L]; // ie 32K scans of 16 channels, or 64K scans of 8 differential channels.

        public UInt32 BufA;
        public UInt32 BufB;
        public bool bGoing;
        public UInt32 KnownBytesLeft;
        public int NextSample, NextChannel;

        public Form1()
        {
            InitializeComponent();
        }
        
        private void Form1_Load(object sender, EventArgs e)
        {
            // Called before Form is displayed Initialize resources:
            
            // Initialize default Device Only:
            DeviceIndex = AIOUSB.diOnly; 
           
            // Device data:
            UInt32 Status;
            UInt32 PID = 0;
            UInt32 NameSize = 256;
            string strName = "name";
            UInt32 DIOBytes = 0;
            UInt32 Counters = 0;
            UInt64 SerNum;

            UInt32 ERROR_SUCCESS = 0;
            bool deviceIndexValid = false;

            
            // Get The Device Information test for valid device found:
            Status = AIOUSB.QueryDeviceInfo(DeviceIndex, out PID, out NameSize, out strName, out DIOBytes, out Counters);
            if ((Status == ERROR_SUCCESS) && ((PID >= 0x8040) && (PID <= 0x815D))) // AI and AIO
            {
                deviceIndexValid = true;
            }
            else
            {
                // If Only device is not valid then Launch connect device dialog box:
                // New parent aware subform:
                FormDetect DetectSubForm = new FormDetect(this);
                DetectSubForm.ShowDialog();

                Status = AIOUSB.QueryDeviceInfo(DeviceIndex, out PID, out NameSize, out strName, out DIOBytes, out Counters);
                if (Status == ERROR_SUCCESS && PID >= 0x8040 && PID <= 0x815D)
                    deviceIndexValid = true;
            }

            if (!deviceIndexValid)
            {
                // No valid device found should exit 
                // this.Close();  // run GUI for demo purposes
            }

            // Check device status:
            Status = AIOUSB.GetDeviceSerialNumber(DeviceIndex, out SerNum);
            

            // FIT--------------------------------------------------------------------------------------------
            // Init config values:

            // Set values:
            for (int i = 0; i < numAnalogOutputs; i++)
            {
                Config.ChannelRange[i] = 1;  // testing: range 1 ==> +-10v 
            } 
            Config.StartStopCH = 0xF0; //Select all 16 channels.	        
            Config.Oversample = 7;//Oversample +7. Max oversample for BG timer acq is +30  


            // Temp copy struct to array for dll call:
            for (int i = 0; i < numAnalogOutputs; i++)
            {
                configArray[i] = Config.ChannelRange[i];
            }
            configArray[18] = Config.StartStopCH;
            configArray[19] = Config.Oversample;

            double Hz = 100;
            uint BlockIndex = 0;
            
            // FIT--------------------------------------------------------------------------------------------
            // Setup Board:
            Status = AIOUSB.ADC_SetCal(DeviceIndex, ":AUTO:");
            Status = AIOUSB.ADC_SetConfig(DeviceIndex, configArray, out ConfigBufSize);
            Status = AIOUSB.ADC_CSA_InitFastScanV(DeviceIndex);
            Status = AIOUSB.CTR_StartOutputFreq(DeviceIndex, BlockIndex, out Hz);

            NextChannel = 0;
            
        }
        
        private String FormatVolt(int Count)
        {
            string formatString;
            formatString = String.Format("{0,3:X}", Count);
            return (formatString);
        }
        
        private void btnDevice_Click_1(object sender, EventArgs e)
        {
            // Launch connect device dialog box
            // Switch between multiple devices or reconnect

            // New parent aware subform:
            FormDetect DetectSubForm = new FormDetect(this);
            DetectSubForm.ShowDialog();
        }
       

        private void btnExecute_Click(object sender, EventArgs e)
        {
            btnExecute.Enabled = false;
            btnStop.Enabled = true;

            // Update:
            UpdateGUIState();
        }

        private void UpdateGUIState()
        {
            // Read data and set GUI state:

            ulong Status;
	        int ChannelsUsed;
	        double[] Volts = new double[Channels];

            // FIT--------------------------------------------------------------------------------------------
            // Get scan data:
            Status = AIOUSB.ADC_CSA_GetFastScanV(DeviceIndex, Volts);

            if ( Status != 0)//ERROR_SUCCESS )
	        {
                MessageBox.Show("ADC_CSA_GetFastScanV Error!");
	        }
	        else
	        {
			    ChannelsUsed = Channels;

                listBoxVoltage.Items.Clear(); //ResetContent();

		        for ( int I = 0; I < ChannelsUsed; ++I )
		        {
                    // Format and display:
                    string strChVolt;
                    string formatString = "Ch {0,2:D2}: {1,8:F4}";
                    strChVolt = String.Format(formatString, I, Volts[I]);

                    listBoxVoltage.Items.Add(strChVolt); //AddString(ChannelData);
		        }
	        }
            
        }

        private void btnStop_Click(object sender, EventArgs e)
        {            
            btnExecute.Enabled = true;
            btnStop.Enabled = false;
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            // FIT--------------------------------------------------------------------------------------------
            // Reset board setings:

            ulong Status;

            //Stop the counter.
            double Hz = 0;
            UInt32 BlockIndex = 0;

            Status = AIOUSB.CTR_StartOutputFreq(DeviceIndex, BlockIndex, out Hz);            
            
            // puts things back the way they were before ADC_InitFastITScanV.
            Status = AIOUSB.ADC_CSA_ResetFastScanV(DeviceIndex);
        }

    }
}
